@@ -123,6 +123,16 @@ impl Context {
         }
     }
 
+    #[cfg(not(target_arch = "wasm32"))]
+    pub unsafe fn device_as_hal<A: wgc::hub::HalApi, F: FnOnce(Option<&A::Device>) -> R, R>(
+        &self,
+        device: &Device,
+        hal_device_callback: F,
+    ) -> R {
+        self.0
+            .device_as_hal::<A, F, R>(device.id, hal_device_callback)
+    }
+
     #[cfg(not(target_arch = "wasm32"))]
     pub unsafe fn texture_as_hal<A: wgc::hub::HalApi, F: FnOnce(Option<&A::Texture>)>(
         &self,
@@ -167,13 +177,13 @@ impl Context {
             label: label.unwrap_or_default().to_string(),
             label_key,
         };
-        let sink = sink_mutex.lock();
+        let mut sink = sink_mutex.lock();
         let mut source_opt: Option<&(dyn Error + 'static)> = Some(&error);
         while let Some(source) = source_opt {
             if let Some(wgc::device::DeviceError::OutOfMemory) =
                 source.downcast_ref::<wgc::device::DeviceError>()
             {
-                return sink.handle_error(crate::Error::OutOfMemoryError {
+                return sink.handle_error(crate::Error::OutOfMemory {
                     source: Box::new(error),
                 });
             }
@@ -181,7 +191,7 @@ impl Context {
         }
 
         // Otherwise, it is a validation error
-        sink.handle_error(crate::Error::ValidationError {
+        sink.handle_error(crate::Error::Validation {
             description: self.format_error(&error),
             source: Box::new(error),
         });
@@ -758,6 +768,7 @@ impl crate::Context for Context {
         Ready<Result<(Self::DeviceId, Self::QueueId), crate::RequestDeviceError>>;
     type MapAsyncFuture = native_gpu_future::GpuFuture<Result<(), crate::BufferAsyncError>>;
     type OnSubmittedWorkDoneFuture = native_gpu_future::GpuFuture<()>;
+    type PopErrorScopeFuture = Ready<Option<crate::Error>>;
 
     fn init(backends: wgt::Backends) -> Self {
         Self(wgc::hub::Global::new(
@@ -1011,6 +1022,22 @@ impl crate::Context for Context {
                 let module = parser.parse().unwrap();
                 wgc::pipeline::ShaderModuleSource::Naga(module)
             }
+            #[cfg(feature = "glsl")]
+            ShaderSource::Glsl {
+                ref shader,
+                stage,
+                ref defines,
+            } => {
+                // Parse the given shader code and store its representation.
+                let options = naga::front::glsl::Options {
+                    stage,
+                    defines: defines.clone(),
+                };
+                let mut parser = naga::front::glsl::Parser::default();
+                let module = parser.parse(&options, shader).unwrap();
+
+                wgc::pipeline::ShaderModuleSource::Naga(module)
+            }
             ShaderSource::Wgsl(ref code) => wgc::pipeline::ShaderModuleSource::Wgsl(Borrowed(code)),
         };
         let (id, error) = wgc::gfx_select!(
@@ -1088,15 +1115,20 @@ impl crate::Context for Context {
         use wgc::binding_model as bm;
 
         let mut arrayed_texture_views = Vec::new();
+        let mut arrayed_samplers = Vec::new();
         if device.features.contains(Features::TEXTURE_BINDING_ARRAY) {
             // gather all the array view IDs first
             for entry in desc.entries.iter() {
                 if let BindingResource::TextureViewArray(array) = entry.resource {
                     arrayed_texture_views.extend(array.iter().map(|view| view.id));
                 }
+                if let BindingResource::SamplerArray(array) = entry.resource {
+                    arrayed_samplers.extend(array.iter().map(|sampler| sampler.id));
+                }
             }
         }
         let mut remaining_arrayed_texture_views = &arrayed_texture_views[..];
+        let mut remaining_arrayed_samplers = &arrayed_samplers[..];
 
         let mut arrayed_buffer_bindings = Vec::new();
         if device.features.contains(Features::BUFFER_BINDING_ARRAY) {
@@ -1135,6 +1167,11 @@ impl crate::Context for Context {
                         bm::BindingResource::BufferArray(Borrowed(slice))
                     }
                     BindingResource::Sampler(sampler) => bm::BindingResource::Sampler(sampler.id),
+                    BindingResource::SamplerArray(array) => {
+                        let slice = &remaining_arrayed_samplers[..array.len()];
+                        remaining_arrayed_samplers = &remaining_arrayed_samplers[array.len()..];
+                        bm::BindingResource::SamplerArray(Borrowed(slice))
+                    }
                     BindingResource::TextureView(texture_view) => {
                         bm::BindingResource::TextureView(texture_view.id)
                     }
@@ -1259,6 +1296,7 @@ impl crate::Context for Context {
                 },
                 targets: Borrowed(frag.targets),
             }),
+            multiview: desc.multiview,
         };
 
         let global = &self.0;
@@ -1480,6 +1518,7 @@ impl crate::Context for Context {
             color_formats: Borrowed(desc.color_formats),
             depth_stencil: desc.depth_stencil,
             sample_count: desc.sample_count,
+            multiview: desc.multiview,
         };
         match wgc::command::RenderBundleEncoder::new(&descriptor, device.id, None) {
             Ok(id) => id,
@@ -1529,6 +1568,20 @@ impl crate::Context for Context {
         error_sink.uncaptured_handler = Box::new(handler);
     }
 
+    fn device_push_error_scope(&self, device: &Self::DeviceId, filter: crate::ErrorFilter) {
+        let mut error_sink = device.error_sink.lock();
+        error_sink.scopes.push(ErrorScope {
+            error: None,
+            filter,
+        });
+    }
+
+    fn device_pop_error_scope(&self, device: &Self::DeviceId) -> Self::PopErrorScopeFuture {
+        let mut error_sink = device.error_sink.lock();
+        let scope = error_sink.scopes.pop().unwrap();
+        ready(scope.error)
+    }
+
     fn buffer_map_async(
         &self,
         buffer: &Self::BufferId,
@@ -2005,7 +2058,7 @@ impl crate::Context for Context {
             buffer.id.id,
             offset, size
         )) {
-            self.handle_error_nolabel(&encoder.error_sink, cause, "CommandEncoder::clear_buffer");
+            self.handle_error_nolabel(&encoder.error_sink, cause, "CommandEncoder::fill_buffer");
         }
     }
 
@@ -2168,19 +2221,44 @@ pub(crate) struct SurfaceOutputDetail {
 
 type ErrorSink = Arc<Mutex<ErrorSinkRaw>>;
 
+struct ErrorScope {
+    error: Option<crate::Error>,
+    filter: crate::ErrorFilter,
+}
+
 struct ErrorSinkRaw {
+    scopes: Vec<ErrorScope>,
     uncaptured_handler: Box<dyn crate::UncapturedErrorHandler>,
 }
 
 impl ErrorSinkRaw {
     fn new() -> ErrorSinkRaw {
         ErrorSinkRaw {
+            scopes: Vec::new(),
             uncaptured_handler: Box::from(default_error_handler),
         }
     }
 
-    fn handle_error(&self, err: crate::Error) {
-        (self.uncaptured_handler)(err);
+    fn handle_error(&mut self, err: crate::Error) {
+        let filter = match err {
+            crate::Error::OutOfMemory { .. } => crate::ErrorFilter::OutOfMemory,
+            crate::Error::Validation { .. } => crate::ErrorFilter::Validation,
+        };
+        match self
+            .scopes
+            .iter_mut()
+            .rev()
+            .find(|scope| scope.filter == filter)
+        {
+            Some(scope) => {
+                if scope.error.is_none() {
+                    scope.error = Some(err);
+                }
+            }
+            None => {
+                (self.uncaptured_handler)(err);
+            }
+        }
     }
 }
 
