@@ -15,10 +15,7 @@ use std::{
 pub use belt::StagingBelt;
 pub use device::{BufferInitDescriptor, DeviceExt};
 pub use encoder::RenderEncoder;
-pub use init::{
-    backend_bits_from_env, initialize_adapter_from_env, initialize_adapter_from_env_or_default,
-    power_preference_from_env,
-};
+pub use init::*;
 
 /// Treat the given byte slice as a SPIR-V module.
 ///
